package ch.iet_gibb.heatcalculator.model;

/**
 * Represents the burning of wood chips to produce energy.
 * Each instance of BurnChips calculates the weight in kilograms and volume in cubic meters
 * of wood chips required to produce a specified amount of energy.
 *
 * @author Yusuf Sabir
 * @version 1.0
 * @since 19.02.2024
 */
public class BurnChips extends Burn {
    /**
     * The conversion factor from kilowatt-hours (KWh) to kilograms (kg).
     */
    private static final double KWH_TO_KG_CONVERSION_FACTOR = 3.5;
    /**
     * The conversion factor for buche wood type to cubic meters (m^3).
     */
    private static final double BUCHE_CONVERSION_FACTOR = 280.0;
    /**
     * The conversion factor for tanne wood type to cubic meters (m^3).
     */
    private static final double TANNE_CONVERSION_FACTOR = 230.0;
    /**
     * The conversion factor for gemischt wood type to cubic meters (m^3).
     */
    private static final double GEMISCHT_CONVERSION_FACTOR = 250.0;
    /**
     * The weight of wood chips required in kilograms (kg).
     */
    private double weight;
    /**
     * The volume of wood chips required in cubic meters (m^3).
     */
    private double volume;
    /**
     * Constructs a new BurnChips instance with the specified wood type and energy amount.
     *
     * @param woodType the type of wood chips being burned (buche, tanne, or gemischt)
     * @param KWh the amount of energy required in kilowatt-hours (KWh)
     */
    public BurnChips(WoodTypes woodType, double KWh){
        super(woodType, KWh);
        calculateResults();
    }
    /**
     * Calculates both the weight and volume of wood chips required.
     */
    @Override
    public void calculateResults() {
        setWeightKg();
        setVolumeM3();
    }
    /**
     * Calculates the weight of wood chips required in kilograms (kg).
     */
    @Override
    public void setWeightKg() {
        weight = energy / KWH_TO_KG_CONVERSION_FACTOR;
    }

    @Override
    public void setVolumeM3() {
        switch (woodType) {
            case BUCHE:
                volume = energy / (KWH_TO_KG_CONVERSION_FACTOR * BUCHE_CONVERSION_FACTOR);
                break;
            case TANNE:
                volume = energy / (KWH_TO_KG_CONVERSION_FACTOR * TANNE_CONVERSION_FACTOR);
                break;
            case GEMISCHT:
                volume = energy / (KWH_TO_KG_CONVERSION_FACTOR * GEMISCHT_CONVERSION_FACTOR);
                break;
            default:
                throw new IllegalArgumentException("Unrecognized wood type: " + woodType);
        }
    }
    public double getWeight() {
        return weight;
    }

    public double getVolume() {
        return volume;
    }

    @Override
    public String toString() {
        return  "Holzart: " + woodType + "\n" +
                "Nötige Energie: " + energy + "KWh \n" +
                "Benötigtes Holz:" + "\n"
                + String.format("%.2f", weight) + "kg \n"
                + String.format("%.2f", volume) + "m3 \n";
    }
}