package ch.iet_gibb.heatcalculator.model;

/**
 * Represents the burning of pellets to produce energy.
 * Each instance of BurnPellets calculates the weight in kilograms and volume in cubic meters
 * of pellets required to produce a specified amount of energy.
 *
 * @author Yusuf Sabir
 * @version 1.0
 * @since 19.02.2024
 */
public class BurnPellets extends Burn {
    /**
     * The conversion factor from kilowatt-hours (KWh) to kilograms (kg).
     */
    private static final double KWH_TO_KG_CONVERSION_FACTOR = 4.8;
    /**
     * The conversion factor for pellets to cubic meters (m^3).
     */
    private static final double PELLETS_CONVERSION_FACTOR = 650.0;
    /**
     * The volume of pellets required in cubic meters (m^3).
     */
    private double volume;
    /**
     * The weight of pellets required in kilograms (kg).
     */
    private double weight;

    /**
     * Constructs a new BurnPellets instance with the specified energy amount.
     *
     * @param KWh the amount of energy required in kilowatt-hours (KWh)
     */
    public BurnPellets(double KWh) {
        super(KWh);
        calculateResults();
    }

    public double getVolume() {
        return volume;
    }

    public double getWeight() {
        return weight;
    }

    /**
     * Calculates both the weight and volume of pellets required.
     */
    @Override
    public void calculateResults() {
        setWeightKg();
        setVolumeM3();
    }

    /**
     * Calculates the weight of pellets required in kilograms (kg).
     */
    @Override
    public void setWeightKg() {
        weight = energy / KWH_TO_KG_CONVERSION_FACTOR;
    }

    /**
     * Calculates the volume of pellets required in cubic meters (m^3).
     */
    @Override
    public void setVolumeM3() {
        volume = energy / (KWH_TO_KG_CONVERSION_FACTOR * PELLETS_CONVERSION_FACTOR);
    }

    @Override
    public String toString() {
        return "Nötige Energie: " + energy + "KWh \n" +
                "Benötigtes Holz:" + "\n"
                + String.format("%.2f", weight) + "kg \n"
                + String.format("%.2f", volume) + "m3 \n";
    }
}