package ch.iet_gibb.heatcalculator.model;

/**
 * Represents the burning of wood to produce energy.
 * Each instance of BurnWood calculates the weight in kilograms, volume in cubic meters,
 * and volume in ster of wood required to produce a specified amount of energy.
 *
 * @author Yusuf Sabir
 * @version 1.0
 * @since 19.02.2024
 */
public class BurnWood extends Burn {
    /**
     * The conversion factor from kilowatt-hours (KWh) to kilograms (kg).
     */
    private static final double KWH_TO_KG_CONVERSION_FACTOR = 4.0;
    /**
     * The conversion factor for buche wood type to cubic meters (m^3).
     */
    private static final double BUCHE_CONVERSION_FACTOR = 500.0;
    /**
     * The conversion factor for tanne wood type to cubic meters (m^3).
     */
    private static final double TANNE_CONVERSION_FACTOR = 400.0;
    /**
     * The conversion factor for gemischt wood type to cubic meters (m^3).
     */
    private static final double GEMISCHT_CONVERSION_FACTOR = 425.0;
    /**
     * The conversion factor for volume in cubic meters to ster.
     */
    private static final double STER_CONVERSION_FACTOR = 0.75;
    /**
     * The weight of wood required in kilograms (kg).
     */
    private double woodWeightKg;
    /**
     * The volume of wood required in cubic meters (m^3).
     */
    private double woodVolumeM3;
    /**
     * The volume of wood required in ster.
     */
    private double woodVolumeSter;

    /**
     * Constructs a new BurnWood instance with the specified wood type and energy amount.
     *
     * @param woodType the type of wood being burned (buche, tanne, or gemischt)
     * @param KWh      the amount of energy required in kilowatt-hours (KWh)
     */
    public BurnWood(WoodTypes woodType, double KWh) {
        super(woodType, KWh);
        calculateResults();
    }

    /**
     * Calculates the weight, volume in cubic meters, and volume in ster of wood required.
     */
    @Override
    public void calculateResults() {
        setWeightKg();
        setVolumeM3();
        setVolumeSter();
    }

    /**
     * Calculates the weight of wood required in kilograms (kg).
     */
    @Override
    public void setWeightKg() {
        woodWeightKg = energy / KWH_TO_KG_CONVERSION_FACTOR;
    }

    /**
     * Calculates the volume of wood required in cubic meters (m^3).
     */
    @Override
    public void setVolumeM3() {
        switch (woodType) {
            case BUCHE:
                woodVolumeM3 = energy / (KWH_TO_KG_CONVERSION_FACTOR * BUCHE_CONVERSION_FACTOR);
                break;
            case TANNE:
                woodVolumeM3 = energy / (KWH_TO_KG_CONVERSION_FACTOR * TANNE_CONVERSION_FACTOR);
                break;
            case GEMISCHT:
                woodVolumeM3 = energy / (KWH_TO_KG_CONVERSION_FACTOR * GEMISCHT_CONVERSION_FACTOR);
                break;
        }
    }

    /**
     * Calculates the volume of wood required in ster.
     */
    private void setVolumeSter() {
        switch (woodType) {
            case BUCHE:
                woodVolumeSter = energy / (KWH_TO_KG_CONVERSION_FACTOR * BUCHE_CONVERSION_FACTOR * STER_CONVERSION_FACTOR);
                break;
            case TANNE:
                woodVolumeSter = energy / (KWH_TO_KG_CONVERSION_FACTOR * TANNE_CONVERSION_FACTOR * STER_CONVERSION_FACTOR);
                break;
            case GEMISCHT:
                woodVolumeSter = energy / (KWH_TO_KG_CONVERSION_FACTOR * GEMISCHT_CONVERSION_FACTOR * STER_CONVERSION_FACTOR);
                break;
        }
    }

    public double getWeight() {
        return woodWeightKg;
    }
    @Override
    public String toString() {
        return "Holzart: " + woodType + "\n" +
                "Nötige Energie: " + energy + "KWh \n" +
                "Benötigtes Holz:" + "\n"
                + String.format("%.2f", woodWeightKg) + "kg \n"
                + String.format("%.2f", woodVolumeM3) + "m3 \n"
                + String.format("%.2f", woodVolumeSter) + "ster \n";
    }
}
